; ********************************************************
; ** Name:    s88.asm                                   **
; ** Version: V3.20 vom 97/03/2009                      **
; ** Project:                                           **
; **                                                    **
; ********************************************************
; ** Sven Brandt                                        **
; ********************************************************
; ** Prozessor: Pic 16F627 / Microchip                  **
; **            1 Befehl = 1 us (4 MHz internal Osc)    **
; **                                                    **
; ** IR: RB0 : Clock Input                              **
; **                                                    **
; **                                                    **
; ** Timer: 0 LED flashing / IR-Inputs (65.5ms)         **
; **        1 not used                                  **
; **        2 not used                                  **
; ********************************************************
; ** Description: Light-barrier-detection for the S88   **
; ********************************************************
; Copyright (C) 2003-2008 Sven Brandt

; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License version 2
; as published by the Free Software Foundation.

; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.

; ********************************************************
; History:
; V3.20: entstanden aus 3.08, nach CLK-Flanke schnellste DATA-Ausgabe in ISR
; V3.08: Debug erleichtert ber #define useledfordebug
; V3.07: Polarittserkennung Bus-Verzgerung
; V3.06: Polarittserkennung bercksichtigt IDLE State (PS dauer-High)
; V3.05: Polarittserkennung auch mit ECoS mglich (nur noch PS betrachten)
; V3.04: Fehler bei invertiertem S88-Bus korrigiert
; V3.03: Polarittserkennung fr den S88-Bus
; V3.02: Signals inverted for S88-N
; V3.01: Freeze Funktion: Wenn LED (A7) gegen Masse und B4 = LOW -> Zustnde einfrieren  
; V3.00 (19/12/05) Kommentare korr.
; V3.00: MS88 und GLS88 merged
; V2.12: BOD (config) enabled
; V2.11: polarity of detection inverted, debouncing decreased
; V2.10: adjust to MS88 V1.10
; V2.02: delete NOPs at start, delete spurious register
; V2.01: PIN-allocation for PCB V3.20
; V2.00: created from MS88 V1.03
; ********************************************************

;#define useledfordebug  ; setzen -> LED zeigt S88-Polaritt an,Freeze-Funktion disabled

 LIST p=16f627                       ; Controller
 ERRORLEVEL 0,-302                   ; Errorlevel
 __idlocs h'0320'                    ; PIC-Kennung (Device ID)
 __config b'11111101010100'          ; Programmer-Configuration
;           ++++-+------------ #8: CPD: Data Code Protection bit
;                 |||||| |        >1 = Data memory code protection off
;                 |||||| |         0 = Data memory code protected
;                 +----------- #7: LVP: Low Voltage Programming Enable
;                  ||||| |         1 = RB4/PGM pin has PGM function, low voltage programming enabled
;                  ||||| |        >0 = RB4/PGM is digital I/O, HV on MCLR must be used for programming
;                  +---------- #6: BODEN: Brown-out Detect Enable bit
;                   |||| |        >1 = BOD enabled
;                   |||| |         0 = BOD disabled
;                   +--------- #5: MCLRE: RA5/MCLR pin function select
;                    ||| |         1 = RA5/MCLR pin function is MCLR
;                    ||| |        >0 = RA5/MCLR pin function is digital Input
;                    |+------- #3: PWRTE: Power-up Timer Enable bitd
;                    | | |         1 = PWRT disabled
;                    | | |        >0 = PWRT enabled
;                    | +------ #2: WDTE: Watchdog Timer Enable bit
;                    |   |        >1 = WDT enabled
;                    |   |         0 = WDT disabled
;                    +--++---- #1-#0: RC-Oscillator
;                              #4,1-0: FOSC2:FOSC0: Oscillator Selection bits
;                                 >100 = INTRC oscillator: I/O function on RA6/OSC2/CLKOUT pin, I/O function on RA7/OSC1/CLKIN

#include <p16f627.inc>

; ********************************************************
; **    Files Assignment                                **
; ********************************************************

;[PORTA] -> R_PORTA \                (3)
;        (1)        (2)-> R_DEBOUNCE -> R_DATA -> [DATA_OUT]
;[PORTB] -> R_PORTB /
; (1) if TMR0 IR, during this IR is locked (GLS only) (read_input)
; (2) IR not locked anymore
; (3) if PS is set (in ISR)

R_PORTA        equ  20h          ; for saving PORTA
R_PORTB        equ  21h          ; for saving PORTB
R_DEBOUNCE     equ  22h          ; for debouncing inputs
R_DATA         equ  24h          ; for shifting data to the putput

R_FLAG         equ  25h          ; various flags
                                 ; #0: 0 if LED (A7) = LOW  -> freeze
                                 ; #0: 1 if LED (A7) = HIGH -> no freeze
                                 ; #6: 0 Polaritt original S88
                                 ; #6: 1 Polaritt inverted S88
                                 ; #7: 0 Polarittssuche ongoing
                                 ; #7: 1 Polarittssuche done
R_COUNT_1      equ  26h          ; Register fr Polarittssuche 
R_COUNT_2      equ  27h          ; Register fr Polarittssuche 
R_COUNT_3      equ  28h          ; Register fr Polarittssuche 
R_BUSERKENNUNG equ  29h          ; Register for debouncing S88 Buserkennung

R_DEB1         equ  31h          ; debouncing counter #1
R_DEB2         equ  32h          ; debouncing counter #2
R_DEB3         equ  33h          ; debouncing counter #3
R_DEB4         equ  34h          ; debouncing counter #4
R_DEB5         equ  35h          ; debouncing counter #5
R_DEB6         equ  36h          ; debouncing counter #6
R_DEB7         equ  37h          ; debouncing counter #7
R_DEB8         equ  38h          ; debouncing counter #8
               
W_TEMP         equ  h'40'        ; saving W-Register
STATUS_TEMP    equ  h'41'        ; saving Status-Register

R_FLAG2        equ  h'2a'        ;
 #define       MODUL_GLS      R_FLAG2,0 ; Modul is GLS (use IR Output)
 #define       MODUL_MS       R_FLAG2,1 ; Modul is MS / SMS / WEICH88 

; ********************************************************
; **  Bit Assignment                                    **
; ********************************************************
T_02    equ d'125'               ; 32 ms
                                 ; Timer2 for IR periode (=repetition time)
                                 ; T_02= (Time in us)/(Prescaler x Postscaler)
CNT_DEB_M equ d'1'               ; counter for debouncing:MS88
CNT_DEB_G equ d'150'             ; counter for debouncing:GLS88
                                 ; increase up to 255 for faster detection
                                 ; decrease down to 1 for less interference liability
CNT_S88   equ d'15'              ; counter for S88 recognizion (x 65 ms keine CLK-Flanke -> no S88)

; ********************************************************
; **  Reset Vector                                      **
; ********************************************************

  org  0000h                           ; Position (Reset Vector)
        goto start

; ********************************************************
; **  Interupt Service Routine ISR                      **
; ********************************************************
  org 0004h                            ; Interupt Vector
; ----- IR IB0-Change (negative/positive Edge) ------------
       ; erst einmal DATA-OUT bedienen. W und STATUS not effected
        bcf PORTB,1                    ; .. (remember inverting transistor)
        btfss R_DATA,0                 ; .. wird bcf gleich wieder mit bsf berschrieben, so merkt
         bsf PORTB,1                   ; .. man das gar nicht (Trgheit Transistor)

; ----- Saving STATUS and W -------------------------------
       ; jetzt ist Zeit, die Register zu retten
        movwf W_TEMP                   ; W-Register Retten
        swapf STATUS,W                 ; Status-Reg. -> w
        movwf STATUS_TEMP              ; Status-Reg. Retten
        bcf STATUS,RP0                 ; -> BANK 0
        bcf PORTA,6                    ; IR-LED OFF

       ; springen in anderen ISR-Teil, wenn S88-Bus invertiert anliegt
        btfsc R_FLAG,6                 ; andere ISR, wenn S88 invertiert anliegt
         goto s88_inverted             ; .. PS invertiert)
        
; ********************************************************
s88_normal
        ; xx us after positive Edge of CLK
        btfss PORTB,7                  ; if PS = HIGH
         goto s88_normal_2

        ; *** if PS = HIGH (=S88-HIGH) then load R_DEBOUNCE into R_DATA ***
        movf R_DEBOUNCE,W              ; load input-values
        movwf R_DATA                   ; .. into R_DATA

        ; *** Output Bit #0 to Data-Out ***
        ; dies ist nur dann interessant, wenn PS anliegt. Dann ist aber genug Zeit,
        ; .. um jetzt die "neuen" Daten noch auf den Bus zu legen.
        btfss R_DATA,0                 ; Output of R_DATA Bit#0
         bsf PORTB,1                   ; .. (remember inverting transistor)
        btfsc R_DATA,0                 ; .. 
         bcf PORTB,1                   ; .. to DATA_OUT

s88_normal_2
       ; warten auf die fallenden CLK-Flanke
       ; clrwdt  ; ntig? geht das in der ISR berhaupt? Knnte fr ECOS interessant sein,
                 ; ..da die ECOS sehr lange im CLK-HIGH Status ist (bis zu 1.2 ms)
        btfsc PORTB,0                  ; wait for CLK gets LOW
         goto $-1                      ; ... (negative Edge)

        ; jetzt Register laden und Flag lschen. schafft auch etwas Zeit bis zum Einlesen der Daten
        movlw CNT_S88
        movwf R_BUSERKENNUNG
        bcf INTCON,INTF                ; clear IR-Flag

        ; jetzt Daten einlesen so ca. 4-5 us nach der fallenden CLK-Flanke
        bsf STATUS,C                   ; Bit am Eingang DATA-IN zum
        btfss PORTB,6                  ; .. Nachschieben in 
         bcf STATUS,C                  ; .. Schieberegister in C
        rrf R_DATA,F                   ; Daten nachschieben
      
; ----- Restore STATUS and W ---------------------
        swapf STATUS_TEMP,W            ; in w-Register
        movwf STATUS                   ; w -> Status
        SWAPF W_TEMP,F                 ; W_TEMP -> W, ohne..
        SWAPF W_TEMP,W                 ; .. Z-Flag zu aendern

        retfie


; ********************************************************
s88_inverted
        ; xx us after positive Edge of CLK
        btfsc PORTB,7                  ; if PS = LOW (=S88-HIGH)
         goto s88_inverted_2

        ; *** if PS = LOW (=S88-HIGH) then load R_DEBOUNCE into R_DATA ***
        movf R_DEBOUNCE,W              ; load input-values
        movwf R_DATA                   ; .. into R_DATA

        ; *** Output Bit #0 to Data-Out ***
        ; dies ist nur dann interessant, wenn PS anliegt. Dann ist aber genug Zeit,
        ; .. um jetzt die "neuen" Daten noch auf den Bus zu legen.
        btfss R_DATA,0                 ; Output of R_DATA Bit#0
         bsf PORTB,1                   ; .. 
        btfsc R_DATA,0                 ; .. 
         bcf PORTB,1                   ; .. to DATA_OUT (inverted)

s88_inverted_2
       ; warten auf die steigende CLK-Flanke
       ; clrwdt  ; ntig? geht das in der ISR berhaupt? Knnte fr ECOS interessant sein,
                 ; ..da die ECOS sehr lange im CLK-HIGH Status ist (bis zu 1.2 ms)
        btfss PORTB,0                  ; wait for CLK gets HIGH
         goto $-1                      ; ... (positive Edge)

        ; jetzt Register laden und Flag lschen. schafft auch etwas Zeit bis zum Einlesen der Daten
        movlw CNT_S88
        movwf R_BUSERKENNUNG
        bcf INTCON,INTF                ; clear IR-Flag

        ; jetzt Daten einlesen so ca. 4-5 us nach der fallenden CLK-Flanke
        bsf STATUS,C                   ; Bit am Eingang DATA-IN zum
        btfsc PORTB,6                  ; .. Nachschieben in 
         bcf STATUS,C                  ; .. Schieberegister in C
        rrf R_DATA,F                   ; Daten nachschieben (invertiert)
      
; ----- Restore STATUS and W ---------------------
        swapf STATUS_TEMP,W            ; in w-Register
        movwf STATUS                   ; w -> Status
        SWAPF W_TEMP,F                 ; W_TEMP -> W, ohne..
        SWAPF W_TEMP,W                 ; .. Z-Flag zu aendern

        retfie


; ********************************************************
; **  Programm                                          **
; ********************************************************
 org   0060h                           ; Position des Programms

start   clrf R_PORTA
        clrf R_PORTB
        clrf R_DEBOUNCE
        clrf R_DATA
        clrf R_FLAG
        clrf R_FLAG2
        clrf R_COUNT_1
        clrf R_COUNT_2
        clrf R_COUNT_3
        incf R_COUNT_3,F    ; = 1, damit ist Pre-Counter sofort "scharf"
        clrf R_BUSERKENNUNG

        clrf R_DEB1
        clrf R_DEB2
        clrf R_DEB3
        clrf R_DEB4
        clrf R_DEB5
        clrf R_DEB6
        clrf R_DEB7
        clrf R_DEB8

        clrf W_TEMP
        clrf STATUS_TEMP

; ********************************************************
; **  Konfiguration PORTS                               **
; ********************************************************
        bsf STATUS,RP0                 ; -> BANK 1
         movlw   B'11111111'
         movwf TRISA                   ; TRISA-Reg. konfigurieren
         movwf TRISB                   ; TRISB-Reg. konfigurieren
         bcf TRISB,1                   ; Output DATA_OUT
        bcf STATUS,RP0                 ; -> BANK 0

        ; ********************
        ; *** check FREEZE ***
        ; ********************
#ifndef useledfordebug
        btfsc PORTA,7                  ; Abfrage, ob FREEZE-Funktion (konfigurierbar ber LED)
         bsf R_FLAG,0                  ; .. konfiguriert ist (fr Stromsensor SMS88 wichtig)
        btfss PORTA,7                  ; wenn FREEZE, dann wird Flag R_FLAG #0 gelscht
         bcf R_FLAG,0                  ; .. ansonsten gesetzt (FLAG entspricht somit PORTA7-Pegel)
#endif
        bsf STATUS,RP0                 ; -> BANK 1
         bcf TRISA,7                   ; Output LED (FREEZE-Konfiguration ist eingelesen) 
        bcf STATUS,RP0                 ; -> BANK 0

        ; ******************
        ; *** check Mode ***
        ; ******************
        bcf MODUL_MS
        bcf MODUL_GLS
        btfss PORTA,6 ; GLS88-Mode
         bsf MODUL_GLS
        btfsc PORTA,6 ; MS88-Mode
         bsf MODUL_MS

        ; *** A6 auf Output setzten @ GLS ***
        btfss MODUL_GLS ; GLS-Mode
         goto port_01
        bsf STATUS,RP0                 ; -> BANK 1
         bcf TRISA,6                   ; Output IR-LED/Mode-Pin
        bcf STATUS,RP0                 ; -> BANK 0

port_01
        clrf PORTA                     ; clear PORTA
        clrf PORTB                     ; clear PORTB

; ********************************************************
; **  Konfiguration COMPARATOR                          **
; ********************************************************

        movlw   B'00000111'
;                 +--------- #7: C2OUT: Comparator 2 output
;                  |||||||       When C2INV=0  1 = C2 VIN+ > C2 VIN
;                  |||||||                     0 = C2 VIN+ < C2 VIN
;                  |||||||       When C2INV=1  0 = C2 VIN+ > C2 VIN
;                  |||||||                     1 = C2 VIN+ < C2 VIN
;                  +-------- #6: C1OUT: Comparator 1 output
;                   ||||||       When C1INV=0  1 = C1 VIN+ > C1 VIN
;                   ||||||                     0 = C1 VIN+ < C1 VIN
;                   ||||||       When C1INV=1  0 = C1 VIN+ > C1 VIN
;                   ||||||                     1 = C1 VIN+ < C1 VIN
;                   +------- #5: C2INV: Comparator 2 output inversion
;                    |||||       1 = C2 Output inverted
;                    |||||       0 = C2 Output not inverted
;                    +------ #4: C1INV: Comparator 1 output inversion
;                     ||||       1 = C1 Output inverted
;                     ||||       0 = C1 Output not inverted
;                     +----- #3: CIS: Comparator Input Switch
;                      +++-- #2-#0: CM2:CM0: Comparator mode
;                               >111 = Comparators Off

       movwf CMCON                     ; CMCON-Reg. konfigurieren

; ********************************************************
; **  Konfiguration TIMER / WDT / Pull-Ups              **
; ********************************************************

        movlw   B'11000111'
;                 +--------- #7: RBPU: PORTB Pull-up Enable bit
;                  |||||        >1 = PORTB pull-ups are disabled
;                  |||||         0 = PORTB pull-ups are enabled (by individual port latch values)
;                  +-------- #6: INTEDG: Interrupt Edge Select bit
;                   ||||        >1 = Interrupt on rising edge of RB0/INT pin (@ S88 not inverted)
;                   ||||      (>)0 = Interrupt on falling edge of RB0/INT pin (@ S88 inverted)
;                   +------- #5: T0CS: TMR0 Clock Source Select bit
;                    |||         1 = Transition on RA4/T0CKI pin
;                    |||        >0 = Internal instruction cycle clock (CLKOUT)
;                    +------ #4: T0SE: TMR0 Source Edge Select bit
;                     ||         1 = Increment on high-to-low transition on RA4/T0CKI pin
;                     ||        >0 = Increment on low-to-high transition on RA4/T0CKI pin
;                     +----- #3: PSA: Prescaler Assignment bit
;                      |         1 = Prescaler assigned to the WDT
;                      |        >0 = Prescaler assigned to TMR0
;                      +++-- #2-#0: PS2:PS0: Prescaler Rate Select bits
;                                 Bit |TMR0 Rate|WDT Rate
;                                 ----+---------+--------
;                                 000 | 1 : 2   | 1 : 1
;                                 001 | 1 : 4   | 1 : 2
;                                 010 | 1 : 8   | 1 : 4
;                                 011 | 1 : 16  | 1 : 8
;                                 100 | 1 : 32  | 1 : 16
;                                 101 | 1 : 64  | 1 : 32
;                                 110 | 1 : 128 | 1 : 64
;                               > 111 | 1 : 256 | 1 : 128

        bsf STATUS,RP0                 ; -> BANK 1
        movwf OPTION_REG               ; OPTION-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0


        movlw   B'01111110'
;                 +--------- #7: unimplemented
;                  ++++----- #6-#3: TOUTPS3:TOUTPS0: Timer2 Output Postscale Select bits
;                      | |       0000 = 1:1 Postscale
;                      | |      >1111 = 1:16 Postscale
;                      +---- #2: TMR2ON: Timer2 On bit
;                       ||      >1 = Timer2 is on
;                       ||       0 = Timer2 is off
;                       ++-- #1-#0: PS2:PS0: Prescaler Rate Select bits
;                                00 = Prescaler is 1
;                                01 = Prescaler is 4
;                               >1x = Prescaler is 16

        movwf T2CON                    ; T2CON-Reg. konfigurieren

        movlw T_02
        bsf STATUS,RP0                 ; -> BANK 1
        movwf PR2                      ; PR2-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

; ********************************************************
; **  Konfiguration INTERRUPT                           **
; ********************************************************

        movlw   B'00000000'
;                 +--------- #7: EEIE: EE Write Complete Interrupt Enable Bit
;                  |||||||       1 = Enables the EE write complete interrupt
;                  |||||||      >0 = Disables the EE write complete interrupt
;                  +-------- #6: CMIE: Comparator Interrupt Enable bit
;                   +------- #5: RCIE: USART Receive Interrupt Enable bit
;                    +------ #4: TXIE: USART Transmit Interrupt Enable bit
;                     +----- #3: Unimplemented: Read as 0
;                      +---- #2: CCP1IE: CCP1 Interrupt Enable bit
;                       +--- #1: TMR2IE: TMR2 to PR2 Match Interrupt Enable bit
;                        +-- #0: TMR1IE: TMR1 Overflow Interrupt Enable bit

        bsf STATUS,RP0                 ; -> BANK 1
        movwf PIE1                     ; PIE1-Reg. konfigurieren
        bcf STATUS,RP0                 ; -> BANK 0

        movlw   B'10010000'
;                 +--------- #7: GIE: Global Interrupt Enable bit
;                  |||||||      >1 = Enables all un-masked interrupts
;                  |||||||       0 = Disables all interrupts
;                  +-------- #6: PEIE: Peripherial Interrupt Enable bit
;                   ||||||       1 = Enables all unmasked peripherial interrupts
;                   ||||||      >0 = Disables all peripherial interrupts
;                   +------- #5: T0IE: TMR0 Overflow Interrupt Enable bit
;                    |||||       1 = Enables the TMR0 interrupt
;                    |||||      >0 = Disables the TMR0 interrupt
;                    +------ #4: INTE: RB0/INT Interrupt Enable bit
;                     ||||      >1 = Enables the RB0/INT interrupt
;                     ||||       0 = Disables the RB0/INT interrupt
;                     +----- #3: RBIE: RB Port Change Interrupt Enable bit
;                      |||       1 = Enables the RB port change interrupt
;                      |||      >0 = Disables the RB port change interrupt
;                      +---- #2: T0IF: TMR0 overflow interrupt flag bit
;                       ||       1 = TMR0 has overflowed (must be cleared in software)
;                       ||      >0 = TMR0 did not overflow
;                       +--- #1: INTF: RB0/INT External Interrupt Flag bit
;                        |       1 = The RB0/INT external interrupt occurred (must be cleared in software)
;                        |      >0 = The RB0/INT interrupt did not occur
;                        +-- #0: RBIF: RB Port Change Interrupt Flag bit
;                                1 = When at least one of the RB7:RB4 pins
;                                    changed state (must be cleared in software)
;                               >0 = None of the RB7:RB4 pins have changed state

        movwf INTCON                   ; INTCON-Reg. konfigurieren

        decfsz R_DEBOUNCE,F            ; wait for Bus stabilisation (~500 us)
         goto $-1                      ; (wenn PS-Signal verzgert kommt, kann falsche Polaritt erkannt werden.
                                       ; Insbesondere am S88-N wenn Stecker gesteckt wird))

; ********************************************************
; **  Main Loop                                         **
; ********************************************************
loop    clrwdt                         ; Watchdog-Timer loeschen

#ifdef useledfordebug ; LED zeig Polaritt des S88-Bus an (Debug)
        btfsc PORTA,7
         bcf PORTA,7
        btfss PORTA,7
         bsf PORTA,7
#endif

        ; *** check for Timer0 (65.5 ms) ***
        btfsc INTCON,T0IF              ; check IR-Flag Timer 0
         call timer0_irf               ; .. if IR-FLAG is set then jump
    
        ; *** Polarittssuche (nur solange bis Polaritt erkannt wurde) ***
       ; btfss R_FLAG,7                 ; Routine "check Polarity", solange R_FLAG#7
         call check_polarity           ; .. nicht gesetzt ist
                
        ; *** Freeze Abfrage ***
        btfsc R_FLAG,0                 ; if R_FLAG#0 = 1, then no freeze function
         goto no_freeze
        btfss PORTB,4                  ; FREEZE konfiguriert -> Abfrage von Port B4, ob dort
         goto loop                     ; .. das FREEZE-Signal anliegt. Wenn nicht -> weiter mit "no_freeze"

no_freeze
        btfsc MODUL_GLS ; GLS88-Mode
         call read_input_gls           ; .. if a#6 is low, then gls88

        btfsc MODUL_MS  ; MS88-Mode
         call read_input_ms            ; .. if a#6 is high, then ms88

        goto loop               


; ********************************************************
; **  SUB - Routines                                    **
; ********************************************************
;*********************************
;** Routine:check_polartity      **
;*********************************
check_polarity
        movf R_BUSERKENNUNG,F          ; wenn R_BUSERKENNUNG = 0
        btfsc STATUS,Z
         return                        ; .. dann zurck, S88 liegt nicht an, Idle

        decf R_COUNT_3,F               ; pre-counter
        btfss STATUS,Z                 ; ca. 45us x 30 x 255 = 350 ms = minimale Betrachtungszeit
         return
        movlw d'30'                    ; reload
        movwf R_COUNT_3                ; Counter R_COUNT_3

        ; ***  Polarittserkennnung S88: sind die Bussignale invertiert (altes S88-P Modul) oder nicht (S88-N) ? ***
        btfss PORTB,7                  ; check PS Signal
         goto PS_LOW


PS_HIGH ; invertiert
        incf R_COUNT_1,F
        btfss STATUS,Z                 ; 255x HIGH gesehen
         return
        bsf R_FLAG,6                   ; Polaritts-Flag setzen (S88 invertiert)
        bsf R_FLAG,7                   ; Polarittssuche abgeschlossen
        clrf R_COUNT_1
        clrf R_COUNT_2

        bcf INTCON,GIE                 ; Interrupt off
        bsf STATUS,RP0                 ; -> BANK 1
         bcf OPTION_REG, INTEDG        ; IR auf die fallende edge
        bcf STATUS,RP0                 ; -> BANK 0
        bsf INTCON,GIE                 ; Interrupt on

        return

PS_LOW  ; normal
        incf R_COUNT_2,F
        btfss STATUS,Z                 ; 255x LOW gesehen
         return
        bcf R_FLAG,6                   ; Polaritts-Flag nicht setzen (S88 nicht invertiert)
        bsf R_FLAG,7                   ; Polarittssuche abgeschlossen
        clrf R_COUNT_1
        clrf R_COUNT_2

        return

;*********************************
;** Routine:timer0_irf          **
;*********************************
timer0_irf
        bcf INTCON,T0IF                ; clear IR-Flag

        movf R_BUSERKENNUNG,F          ; wenn R_BUSERKENNUNG = 0
        btfsc STATUS,Z
         goto led_blink                ; .. dann Blinken

        ; *** S88 liegt an ***         
#ifndef useledfordebug
        bcf PORTA,7                    ; LED continuous ON
        btfss R_FLAG,0                 ; if R_FLAG#0 = 0, then invert LED output
         bsf PORTA,7                   ; LED continuous ON
#endif

        decf R_BUSERKENNUNG,F         
        return

led_blink              
        ; *** S88 liegt NICHT an ***         
#ifndef useledfordebug
        comf PORTA,W
        andlw B'10000000'              ; LED flashing A7
        movwf PORTA
#endif

        return

;*********************************
;** Routine:read_input_gls      **
;*********************************
read_input_gls
        btfss PIR1,TMR2IF              ; check IR-Flag Timer 2
         return
        bcf PIR1,TMR2IF                ; clear IR-Flag

; **** Load Ports into Registers ****
        bsf PORTA,6                    ; IR-LED ON
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         nop                           ; wait..
         movf PORTA,W
         movwf R_PORTA
         movf PORTB,W
         movwf R_PORTB

        bcf PORTA,6                    ; IR-LED OFF

; **** Debounce X1 - Bit #0 ****
check_gls_1 btfsc R_PORTA,2             ; Port low -> no train -> Bit clear
        goto check_gls_1h
check_gls_1l;*** PORT is low ***
        movlw CNT_DEB_G
        subwf R_DEB1,F
        btfss STATUS,C                 ; check for CARRY
         goto check_gls_1n             ; if R_DEB1 < 0 then jump
        goto check_gls_2               ; else next
check_gls_1h;*** PORT is high ***
        movlw CNT_DEB_G
        addwf R_DEB1,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_gls_1y             ; if R_DEB1 > 255 then jump
        goto check_gls_2               ; else next
check_gls_1y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB1                   ; .. to FF
        bsf R_DEBOUNCE,0               ;
        goto check_gls_2               ; then  next
check_gls_1n;*** INPUT is not active *** 
        clrf R_DEB1                    ; clear counter
        bcf R_DEBOUNCE,0                  ;
        goto check_gls_2               ; then  next

; **** Debounce X2 - Bit #1 ****
check_gls_2 btfsc R_PORTA,3            ; Port low -> no train -> Bit clear
        goto check_gls_2h
check_gls_2l;*** PORT is low ***
        movlw CNT_DEB_G
        subwf R_DEB2,F
        btfss STATUS,C                 ; check for CARRY
         goto check_gls_2n             ; if R_DEB2 < 0 then jump
        goto check_gls_3               ; else next
check_gls_2h;*** PORT is high ***
        movlw CNT_DEB_G
        addwf R_DEB2,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_gls_2y             ; if R_DEB2 > 255 then jump
        goto check_gls_3               ; else next
check_gls_2y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB2                   ; .. to FF
        bsf R_DEBOUNCE,1               ;
        goto check_gls_3               ; then  next
check_gls_2n;*** INPUT is not active *** 
        clrf R_DEB2                    ; clear counter
        bcf R_DEBOUNCE,1                  ;
        goto check_gls_3               ; then  next

; **** Debounce X3 - Bit #2 ****
check_gls_3 btfsc R_PORTB,2            ; Port low -> no train -> Bit clear
        goto check_gls_3h
check_gls_3l;*** PORT is low ***
        movlw CNT_DEB_G
        subwf R_DEB3,F
        btfss STATUS,C                 ; check for CARRY
         goto check_gls_3n             ; if R_DEB3 < 0 then jump
        goto check_gls_4               ; else next
check_gls_3h;*** PORT is high ***
        movlw CNT_DEB_G
        addwf R_DEB3,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_gls_3y             ; if R_DEB3 > 255 then jump
        goto check_gls_4               ; else next
check_gls_3y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB3                   ; .. to FF
        bsf R_DEBOUNCE,2               ;
        goto check_gls_4               ; then  next
check_gls_3n;*** INPUT is not active *** 
        clrf R_DEB3                    ; clear counter
        bcf R_DEBOUNCE,2                  ;
        goto check_gls_4               ; then  next


; **** Debounce X4 - Bit #3 ****
check_gls_4 btfsc R_PORTB,3            ; Port low -> no train -> Bit clear
        goto check_gls_4h
check_gls_4l;*** PORT is low ***
        movlw CNT_DEB_G
        subwf R_DEB4,F
        btfss STATUS,C                 ; check for CARRY
         goto check_gls_4n             ; if R_DEB4 < 0 then jump
        goto check_gls_5               ; else next
check_gls_4h;*** PORT is high ***
        movlw CNT_DEB_G
        addwf R_DEB4,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_gls_4y             ; if R_DEB4 > 255 then jump
        goto check_gls_5               ; else next
check_gls_4y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB4                   ; .. to FF
        bsf R_DEBOUNCE,3               ;
        goto check_gls_5               ; then  next
check_gls_4n;*** INPUT is not active *** 
        clrf R_DEB4                    ; clear counter
        bcf R_DEBOUNCE,3                  ;
        goto check_gls_5               ; then  next

; **** Debounce X5 - Bit #4 ****
check_gls_5 btfsc R_PORTA,1            ; Port low -> no train -> Bit clear
        goto check_gls_5h
check_gls_5l;*** PORT is low ***
        movlw CNT_DEB_G
        subwf R_DEB5,F
        btfss STATUS,C                 ; check for CARRY
         goto check_gls_5n             ; if R_DEB5 < 0 then jump
        goto check_gls_6               ; else next
check_gls_5h;*** PORT is high ***
        movlw CNT_DEB_G
        addwf R_DEB5,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_gls_5y             ; if R_DEB5 > 255 then jump
        goto check_gls_6               ; else next
check_gls_5y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB5                   ; .. to FF
        bsf R_DEBOUNCE,4               ;
        goto check_gls_6               ; then  next
check_gls_5n;*** INPUT is not active *** 
        clrf R_DEB5                    ; clear counter
        bcf R_DEBOUNCE,4                  ;
        goto check_gls_6               ; then  next

; **** Debounce X6 - Bit #5 ****
check_gls_6 btfsc R_PORTA,0            ; Port low -> no train -> Bit clear
        goto check_gls_6h
check_gls_6l;*** PORT is low ***
        movlw CNT_DEB_G
        subwf R_DEB6,F
        btfss STATUS,C                 ; check for CARRY
         goto check_gls_6n             ; if R_DEB6 < 0 then jump
        goto check_gls_7               ; else next
check_gls_6h;*** PORT is high ***
        movlw CNT_DEB_G
        addwf R_DEB6,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_gls_6y             ; if R_DEB6 > 255 then jump
        goto check_gls_7               ; else next
check_gls_6y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB6                   ; .. to FF
        bsf R_DEBOUNCE,5               ;
        goto check_gls_7               ; then  next
check_gls_6n;*** INPUT is not active *** 
        clrf R_DEB6                    ; clear counter
        bcf R_DEBOUNCE,5                  ;
        goto check_gls_7               ; then  next

; **** Debounce X7 - Bit #6 ****
check_gls_7 btfsc R_PORTB,5            ; Port low -> no train -> Bit clear
        goto check_gls_7h
check_gls_7l;*** PORT is low ***
        movlw CNT_DEB_G
        subwf R_DEB7,F
        btfss STATUS,C                 ; check for CARRY
         goto check_gls_7n             ; if R_DEB7 < 0 then jump
        goto check_gls_8               ; else next
check_gls_7h;*** PORT is high ***
        movlw CNT_DEB_G
        addwf R_DEB7,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_gls_7y             ; if R_DEB7 > 255 then jump
        goto check_gls_8               ; else next
check_gls_7y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB7                   ; .. to FF
        bsf R_DEBOUNCE,6               ;
        goto check_gls_8               ; then  next
check_gls_7n;*** INPUT is not active *** 
        clrf R_DEB7                    ; clear counter
        bcf R_DEBOUNCE,6                  ;
        goto check_gls_8               ; then  next

; **** Debounce X8 - Bit #7 ****
check_gls_8 btfsc R_PORTA,4            ; Port low -> no train -> Bit clear
        goto check_gls_8h
check_gls_8l;*** PORT is low ***
        movlw CNT_DEB_G
        subwf R_DEB8,F
        btfss STATUS,C                 ; check for CARRY
         goto check_gls_8n             ; if R_DEB8 < 0 then jump
        goto check_gls_9               ; else next
check_gls_8h;*** PORT is high ***
        movlw CNT_DEB_G
        addwf R_DEB8,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_gls_8y             ; if R_DEB8 > 255 then jump
        goto check_gls_9               ; else next
check_gls_8y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB8                   ; .. to FF
        bsf R_DEBOUNCE,7               ;
        goto check_gls_9               ; then  next
check_gls_8n;*** INPUT is not active *** 
        clrf R_DEB8                    ; clear counter
        bcf R_DEBOUNCE,7                  ;
        goto check_gls_9               ; then  next

check_gls_9 return


;*********************************
;** Routine:read_input_ms       **
;*********************************
read_input_ms
        bcf PIR1,TMR2IF                ; clear IR-Flag

; **** Load Ports into Registers ****
        movf PORTA,W
        movwf R_PORTA
        movf PORTB,W
        movwf R_PORTB

; **** Debounce X1 - Bit #0 ****
check_ms_1 btfsc R_PORTA,4             ; Port high -> no train -> Bit clear
        goto check_ms_1h
check_ms_1l;*** PORT is low ***
        movlw CNT_DEB_M
        addwf R_DEB1,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_ms_1y              ; if R_DEB1 > 255 then jump
        goto check_ms_2                ; else next
check_ms_1h;*** PORT is high ***
        movlw CNT_DEB_M
        subwf R_DEB1,F
        btfss STATUS,C                 ; check for CARRY
         goto check_ms_1n              ; if R_DEB1 < 0 then jump
        goto check_ms_2                ; else next
check_ms_1y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB1                   ; .. to FF
        bsf R_DEBOUNCE,0               ;
        goto check_ms_2                ; then  next
check_ms_1n;*** INPUT is not active *** 
        clrf R_DEB1                    ; clear counter
        bcf R_DEBOUNCE,0                  ;
        goto check_ms_2                ; then  next

; **** Debounce X2 - Bit #1 ****
check_ms_2 btfsc R_PORTB,5             ; Port high -> no train -> Bit clear
        goto check_ms_2h
check_ms_2l;*** PORT is low ***
        movlw CNT_DEB_M
        addwf R_DEB2,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_ms_2y              ; if R_DEB2 > 255 then jump
        goto check_ms_3                ; else next
check_ms_2h;*** PORT is high ***
        movlw CNT_DEB_M
        subwf R_DEB2,F
        btfss STATUS,C                 ; check for CARRY
         goto check_ms_2n              ; if R_DEB2 < 0 then jump
        goto check_ms_3                ; else next
check_ms_2y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB2                   ; .. to FF
        bsf R_DEBOUNCE,1               ;
        goto check_ms_3                ; then  next
check_ms_2n;*** INPUT is not active *** 
        clrf R_DEB2                    ; clear counter
        bcf R_DEBOUNCE,1                  ;
        goto check_ms_3                ; then  next

; **** Debounce X3 - Bit #2 ****
check_ms_3 btfsc R_PORTB,3             ; Port high -> no train -> Bit clear
        goto check_ms_3h
check_ms_3l;*** PORT is low ***
        movlw CNT_DEB_M
        addwf R_DEB3,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_ms_3y              ; if R_DEB3 > 255 then jump
        goto check_ms_4                ; else next
check_ms_3h;*** PORT is high ***
        movlw CNT_DEB_M
        subwf R_DEB3,F
        btfss STATUS,C                 ; check for CARRY
         goto check_ms_3n              ; if R_DEB3 < 0 then jump
        goto check_ms_4                ; else next
check_ms_3y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB3                   ; .. to FF
        bsf R_DEBOUNCE,2               ;
        goto check_ms_4                ; then  next
check_ms_3n;*** INPUT is not active *** 
        clrf R_DEB3                    ; clear counter
        bcf R_DEBOUNCE,2                  ;
        goto check_ms_4                ; then  next


; **** Debounce X4 - Bit #3 ****
check_ms_4 btfsc R_PORTB,2             ; Port high -> no train -> Bit clear
        goto check_ms_4h
check_ms_4l;*** PORT is low ***
        movlw CNT_DEB_M
        addwf R_DEB4,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_ms_4y              ; if R_DEB4 > 255 then jump
        goto check_ms_5                ; else next
check_ms_4h;*** PORT is high ***
        movlw CNT_DEB_M
        subwf R_DEB4,F
        btfss STATUS,C                 ; check for CARRY
         goto check_ms_4n              ; if R_DEB4 < 0 then jump
        goto check_ms_5                ; else next
check_ms_4y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB4                   ; .. to FF
        bsf R_DEBOUNCE,3               ;
        goto check_ms_5                ; then  next
check_ms_4n;*** INPUT is not active *** 
        clrf R_DEB4                    ; clear counter
        bcf R_DEBOUNCE,3                  ;
        goto check_ms_5                ; then  next

; **** Debounce X5 - Bit #4 ****
check_ms_5 btfsc R_PORTA,3             ; Port high -> no train -> Bit clear
        goto check_ms_5h
check_ms_5l;*** PORT is low ***
        movlw CNT_DEB_M
        addwf R_DEB5,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_ms_5y              ; if R_DEB5 > 255 then jump
        goto check_ms_6                ; else next
check_ms_5h;*** PORT is high ***
        movlw CNT_DEB_M
        subwf R_DEB5,F
        btfss STATUS,C                 ; check for CARRY
         goto check_ms_5n              ; if R_DEB5 < 0 then jump
        goto check_ms_6                ; else next
check_ms_5y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB5                   ; .. to FF
        bsf R_DEBOUNCE,4               ;
        goto check_ms_6                ; then  next
check_ms_5n;*** INPUT is not active *** 
        clrf R_DEB5                    ; clear counter
        bcf R_DEBOUNCE,4                  ;
        goto check_ms_6                ; then  next

; **** Debounce X6 - Bit #5 ****
check_ms_6 btfsc R_PORTA,2             ; Port high -> no train -> Bit clear
        goto check_ms_6h
check_ms_6l;*** PORT is low ***
        movlw CNT_DEB_M
        addwf R_DEB6,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_ms_6y              ; if R_DEB6 > 255 then jump
        goto check_ms_7                ; else next
check_ms_6h;*** PORT is high ***
        movlw CNT_DEB_M
        subwf R_DEB6,F
        btfss STATUS,C                 ; check for CARRY
         goto check_ms_6n              ; if R_DEB6 < 0 then jump
        goto check_ms_7                ; else next
check_ms_6y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB6                   ; .. to FF
        bsf R_DEBOUNCE,5               ;
        goto check_ms_7                ; then  next
check_ms_6n;*** INPUT is not active *** 
        clrf R_DEB6                    ; clear counter
        bcf R_DEBOUNCE,5                  ;
        goto check_ms_7                ; then  next

; **** Debounce X7 - Bit #6 ****
check_ms_7 btfsc R_PORTA,0             ; Port high -> no train -> Bit clear
        goto check_ms_7h
check_ms_7l;*** PORT is low ***
        movlw CNT_DEB_M
        addwf R_DEB7,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_ms_7y              ; if R_DEB7 > 255 then jump
        goto check_ms_8                ; else next
check_ms_7h;*** PORT is high ***
        movlw CNT_DEB_M
        subwf R_DEB7,F
        btfss STATUS,C                 ; check for CARRY
         goto check_ms_7n              ; if R_DEB7 < 0 then jump
        goto check_ms_8                ; else next
check_ms_7y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB7                   ; .. to FF
        bsf R_DEBOUNCE,6               ;
        goto check_ms_8                ; then  next
check_ms_7n;*** INPUT is not active *** 
        clrf R_DEB7                    ; clear counter
        bcf R_DEBOUNCE,6                  ;
        goto check_ms_8                ; then  next

; **** Debounce X8 - Bit #7 ****
check_ms_8 btfsc R_PORTA,1             ; Port high -> no train -> Bit clear
        goto check_ms_8h
check_ms_8l;*** PORT is low ***
        movlw CNT_DEB_M
        addwf R_DEB8,F
        btfsc STATUS,C                 ; check for CARRY
         goto check_ms_8y              ; if R_DEB8 > 255 then jump
        goto check_ms_9                ; else next
check_ms_8h;*** PORT is high ***
        movlw CNT_DEB_M
        subwf R_DEB8,F
        btfss STATUS,C                 ; check for CARRY
         goto check_ms_8n              ; if R_DEB8 < 0 then jump
        goto check_ms_9                ; else next
check_ms_8y;*** INPUT is active ***
        movlw h'ff'                    ; clear counter
        movwf R_DEB8                   ; .. to FF
        bsf R_DEBOUNCE,7               ;
        goto check_ms_9                ; then  next
check_ms_8n;*** INPUT is not active *** 
        clrf R_DEB8                    ; clear counter
        bcf R_DEBOUNCE,7                  ;
        goto check_ms_9                ; then  next

check_ms_9 return
 
        END

